/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package ic3.sample.component;

import java.util.List;
import java.util.ResourceBundle;

import org.exoplatform.faces.core.component.UIGrid;
import org.exoplatform.faces.core.component.UIPageIterator;
import org.exoplatform.faces.core.component.model.ActionCell;
import org.exoplatform.faces.core.component.model.ActionColumn;
import org.exoplatform.faces.core.component.model.Button;
import org.exoplatform.faces.core.component.model.Column;
import org.exoplatform.faces.core.component.model.ComponentCell;
import org.exoplatform.faces.core.component.model.ListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.Row;
import org.exoplatform.faces.core.component.model.Rows;
import org.exoplatform.faces.core.event.ChangePageEvent;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import ic3.sample.*;
import ic3.sample.model.* ;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIListPatient.java,v 1.1 2004/09/30 18:06:02 tuan08 Exp $
 */
public class UIListPatient extends UIGrid { 
    static String VIEW_ACTION = "view" ;
    static String EDIT_ACTION = "edit" ;
    static String ADD_ACTION = "add" ;

    static Parameter[] VIEW_PARAMS =   { new Parameter(ACTION , VIEW_ACTION)  } ;
	static Parameter[] EDIT_PARAMS =   { new Parameter(ACTION , EDIT_ACTION)  } ;
	static Parameter[] ADD_PARAMS =   { new Parameter(ACTION , ADD_ACTION)  } ;
	
	static private int PAGE_SIZE = 25 ;
	private List patients_ ;
	private UIPageIterator uiPageIterator_ ;
	private MyService service_ ;
	private PatientDataHandler patientDataHandler_ ;
	
	public UIListPatient(MyService service, ResourceBundle res) throws Exception {
		setId("UIListPatient") ;
		setClazz("UIListPatient") ;
		service_ = service  ;
		patients_ = service.getPatients() ;
		uiPageIterator_ = new UIPageIterator(0, PAGE_SIZE) ;
		uiPageIterator_.setAvailablePage(patients_.size(), PAGE_SIZE) ;
		
		patientDataHandler_ = new PatientDataHandler() ;
		add(new Rows(patientDataHandler_, "even", "odd").
				add(new Column(res.getString("header.patient-id"), "patientId")).
				add(new Column(res.getString("header.first-name"), "firstName")).
				add(new Column(res.getString("header.last-name"), "lastName")).
				add(new Column(res.getString("header.email"), "email")).
				add(new ActionColumn(res.getString("header.action"), "patientId").
						add(true, new Button(res.getString("button.view"), VIEW_PARAMS)).
						add(true, new Button(res.getString("button.add"), ADD_PARAMS)).
						add(true, new Button(res.getString("button.edit"), EDIT_PARAMS)))) ;
		
		add(new Row().setClazz("footer").
		        add(new ActionCell().
		            add(true, new Button(res.getString("button.add"), ADD_PARAMS)).
		            addColspan("5").addAlign("center")));
		
		add(new Row().
				add(new ComponentCell(this, uiPageIterator_).
						addColspan("5")));
		
		addActionListener(ViewPatientActionListener.class, VIEW_ACTION) ;
		//addActionListener(new ViewPatientActionListener().
		//                  setActionToListen(VIEW_INFO_ACTION)) ;
		addActionListener(EditPatientActionListener.class, EDIT_ACTION) ;
		addActionListener(ChangePageActionListener.class, ChangePageEvent.ACTION) ;
		addActionListener(AddPatientActionListener.class, ADD_ACTION) ;
		changePage() ;
	}
	
	public UIPageIterator getUIPageIterator() { return uiPageIterator_ ; }
	
	
	public void changePage() { 
		int from = uiPageIterator_.getFrom() ;
		int to = uiPageIterator_.getTo() ;
    patientDataHandler_.setDatas(patients_.subList(from, to)) ;
	}
	
	public void update() throws Exception {
		patients_ = service_.getPatients() ;
		uiPageIterator_.setAvailablePage(patients_.size(), PAGE_SIZE) ;
		int from = uiPageIterator_.getFrom() ;
		int to = uiPageIterator_.getTo() ;
    patientDataHandler_.setDatas(patients_.subList(from, to)) ;
	}
	
	static public class PatientDataHandler extends ListDataHandler {
		private Patient patient_  ;
		
		public String  getData(String fieldName)   {
			if("patientId".equals(fieldName)) return Integer.toString(patient_.getPatientId());
			if("lastName".equals(fieldName)) return patient_.getLastName() ;
			if("firstName".equals(fieldName)) return patient_.getFirstName()  ;
			if("email".equals(fieldName)) return patient_.getEMail() ;
			return null ;
		}
		
		public void setCurrentObject(Object o) { patient_ = (Patient) o; }
	}
	
	
	static public class ViewPatientActionListener extends ExoActionListener  {
		public void execute(ExoActionEvent event) throws Exception {
      UIListPatient uiListPatient = (UIListPatient) event.getSource() ;
			String patientId = event.getParameter("objectId") ;
			System.out.println("execute view patient info action, patient id: " + patientId) ;
			Patient patient = uiListPatient.service_.findPatientByName(patientId) ;
			UIPatientInfo uiPatientInfo = 
			  (UIPatientInfo)uiListPatient.getSibling(UIPatientInfo.class) ;
			uiPatientInfo.setPatient(patient) ;
			uiListPatient.setRenderedSibling(UIPatientInfo.class) ;
		}
	}
	
	static public class EditPatientActionListener extends ExoActionListener  {
		public void execute(ExoActionEvent event) throws Exception {
      UIListPatient uiListPatient = (UIListPatient) event.getSource() ;
			String patientId = event.getParameter("objectId") ;
			System.out.println("execute edit patient info action, patient id: " + patientId) ;
			Patient patient = uiListPatient.service_.findPatientByName(patientId) ;
			UIPatientForm uiPatientForm = 
			  (UIPatientForm)uiListPatient.getSibling(UIPatientForm.class) ;
			uiPatientForm.setPatient(patient) ;
			uiListPatient.setRenderedSibling(UIPatientForm.class) ;
		}
	}

	static public class AddPatientActionListener extends ExoActionListener  {
		public void execute(ExoActionEvent event) throws Exception {
			UIListPatient uiListPatient = (UIListPatient) event.getSource() ;
			UIPatientForm uiPatientForm = 
			  (UIPatientForm)uiListPatient.getSibling(UIPatientForm.class) ;
			uiPatientForm.setPatient(null) ;
			
			uiListPatient.setRenderedSibling(UIPatientForm.class) ;
		}
	}

	static public class ChangePageActionListener extends ExoActionListener {
		public void execute(ExoActionEvent action) throws Exception {
      UIListPatient uiListUser = (UIListPatient) action.getComponent() ;
			ChangePageEvent event = (ChangePageEvent) action ;
			uiListUser.uiPageIterator_.setCurrentPage(event.getSelectPage()) ;
      uiListUser.changePage() ;
		}
	}
}
